@@ -7,7 +7,11 @@ fi
 if [ ! -f /etc/default/netifyd ]; then
     sudo cp etc/default/netifyd /etc/default/
 fi
-sudo cp -r run/* /run/
 sudo cp -r usr/* /usr/
 sudo cp -r var/* /var/
 
+sudo cp etc/cron.d/* /etc/cron.d/
+sudo /etc/init.d/cron restart
+
+sudo /usr/sbin/netifyd --provision
+
