@@ -61,41 +61,43 @@
 #define ND_MAX_UDP_PKTS         8       // Maximum number of UDP packets to process.
 
 #ifndef ND_VOLATILE_STATEDIR
-#define ND_VOLATILE_STATEDIR    "/var/lib/netifyd"
+#define ND_VOLATILE_STATEDIR    "/var/run/netifyd"
 #endif
 
 #ifndef ND_PERSISTENT_STATEDIR
 #define ND_PERSISTENT_STATEDIR  "/etc/netify.d"
 #endif
 
-#ifndef ND_LIBEXECDIR
-#define ND_LIBEXECDIR           "/usr/libexec/netifyd"
-#endif
-
-#ifndef ND_PID_FILE_NAME
-#define ND_PID_FILE_NAME        "/var/run/netifyd/netifyd.pid"
+#ifndef ND_DATADIR
+#define ND_DATADIR              "/usr/share/netifyd"
 #endif
 
 #ifndef ND_CONF_FILE_NAME
 #define ND_CONF_FILE_NAME       "/etc/netifyd.conf"
 #endif
 
+#ifndef ND_PID_FILE_NAME
+#define ND_PID_FILE_NAME        ND_VOLATILE_STATEDIR "/netifyd.pid"
+#endif
+
 #define ND_DNS_CACHE_FILE_NAME  ND_VOLATILE_STATEDIR "/dns-cache.csv"
 
-#define ND_JSON_VERSION         1.5     // JSON format version
+#define ND_JSON_VERSION         1.6     // JSON format version
 #ifndef ND_JSON_FILE_NAME
 #define ND_JSON_FILE_NAME       ND_VOLATILE_STATEDIR "/netifyd.json"
 #endif
 #define ND_JSON_FILE_USER       "root"
 #define ND_JSON_FILE_GROUP      "webconfig"
 #define ND_JSON_FILE_MODE       0640
+#define ND_JSON_FILE_RESPONSE   ND_VOLATILE_STATEDIR "/netifyd-response.json"
 #define ND_JSON_FILE_BAD_SEND   ND_VOLATILE_STATEDIR "/netifyd-bad-send.json"
 #define ND_JSON_FILE_BAD_RECV   ND_VOLATILE_STATEDIR "/netifyd-bad-recv.json"
+#define ND_JSON_DATA_CHUNKSIZ   4096
 
 #define ND_PCAP_SNAPLEN         1536    // Capture snap length
 #define ND_PCAP_READ_TIMEOUT    500     // Milliseconds
 
-#define ND_URL_UPLOAD           "https://sink.netify.ai/v1/"
+#define ND_URL_UPLOAD           "https://sink.netify.ai/v2/"
 #define ND_URL_PROVISION        "https://portal.netify.ai/app/provision/agent?uuid="
 #define ND_COOKIE_JAR           ND_VOLATILE_STATEDIR "/netifyd.cookies"
 #define ND_UPLOAD_TIMEOUT       300     // Default 5-minute upload timeout
@@ -120,8 +122,6 @@
 #define ND_COMPRESS_SIZE       (1024 * 10)
 #define ND_ZLIB_CHUNK_SIZE      16384   // Compress this many bytes at a time
 
-#define ND_FILE_BUFSIZ          4096
-
 #define ND_SOCKET_PORT          "7150"
 #define ND_SOCKET_PATH_MODE     0640
 #define ND_SOCKET_PATH_USER     "root"
@@ -131,9 +131,8 @@
 #define PACKAGE_URL             "http://www.egloo.ca/"
 #endif
 
-#define ND_CONF_CUSTOM_MATCH    ND_PERSISTENT_STATEDIR "/netify-sink.conf"
-#define ND_CONF_CONTENT_MATCH   ND_PERSISTENT_STATEDIR "/app-content-match.csv"
-#define ND_CONF_HOST_MATCH      ND_PERSISTENT_STATEDIR "/app-host-match.csv"
+#define ND_CONF_SINK_BASE       "netify-sink.conf"
+#define ND_CONF_SINK_PATH       ND_PERSISTENT_STATEDIR "/" ND_CONF_SINK_BASE
 
 #define ND_STR_ETHALEN          (ETH_ALEN * 2 + ETH_ALEN - 1)
 
@@ -149,7 +148,7 @@ typedef map<string, string> nd_device_netlink;
 typedef map<string, string> nd_inotify_watch;
 #ifdef _ND_USE_PLUGINS
 class ndPluginLoader;
-typedef map<string, ndPluginLoader *> nd_plugin;
+typedef map<string, ndPluginLoader *> nd_plugins;
 #endif
 
 enum nd_global_flags {
@@ -157,11 +156,11 @@ enum nd_global_flags {
     ndGF_DEBUG_UPLOAD = 0x2,
     ndGF_DEBUG_WITH_ETHERS = 0x4,
     ndGF_DEBUG_DNS_CACHE = 0x8,
-    ndGF_OVERRIDE_CONTENT_MATCH = 0x10,
-    ndGF_OVERRIDE_CUSTOM_MATCH = 0x20,
-    ndGF_OVERRIDE_HOST_MATCH = 0x40,
+    ndGF_OVERRIDE_SINK_CONFIG = 0x10,
+    ndGF_BITHOLE_0x20 = 0x20,
+    ndGF_BITHOLE_0x40 = 0x40,
     ndGF_SSL_USE_TLSv1 = 0x80,
-    ndGF_SSL_VERIFY_PEER = 0x100,
+    ndGF_SSL_VERIFY = 0x100,
     ndGF_USE_CONNTRACK = 0x200,
     ndGF_USE_NETLINK = 0x400,
     ndGF_FREE_0x800 = 0x800,
@@ -178,11 +177,9 @@ enum nd_global_flags {
 #define ND_DEBUG_UPLOAD (nd_config.flags & ndGF_DEBUG_UPLOAD)
 #define ND_DEBUG_WITH_ETHERS (nd_config.flags & ndGF_DEBUG_WITH_ETHERS)
 #define ND_DEBUG_DNS_CACHE (nd_config.flags & ndGF_DEBUG_DNS_CACHE)
-#define ND_OVERRIDE_CONTENT_MATCH (nd_config.flags & ndGF_OVERRIDE_CONTENT_MATCH)
-#define ND_OVERRIDE_CUSTOM_MATCH (nd_config.flags & ndGF_OVERRIDE_CUSTOM_MATCH)
-#define ND_OVERRIDE_HOST_MATCH (nd_config.flags & ndGF_OVERRIDE_HOST_MATCH)
+#define ND_OVERRIDE_SINK_CONFIG (nd_config.flags & ndGF_OVERRIDE_SINK_CONFIG)
 #define ND_SSL_USE_TLSv1 (nd_config.flags & ndGF_SSL_USE_TLSv1)
-#define ND_SSL_VERIFY_PEER (nd_config.flags & ndGF_SSL_VERIFY_PEER)
+#define ND_SSL_VERIFY (nd_config.flags & ndGF_SSL_VERIFY)
 #define ND_USE_CONNTRACK (nd_config.flags & ndGF_USE_CONNTRACK)
 #define ND_USE_NETLINK (nd_config.flags & ndGF_USE_NETLINK)
 #define ND_USE_SINK (nd_config.flags & ndGF_USE_SINK)
@@ -193,11 +190,15 @@ enum nd_global_flags {
 #define ND_REPLAY_DELAY (nd_config.flags & ndGF_REPLAY_DELAY)
 #define ND_REMAIN_IN_FOREGROUND (nd_config.flags & ndGF_REMAIN_IN_FOREGROUND)
 
+#define ND_GF_SET_FLAG(flag, value) \
+{ \
+    if (value) nd_config.flags |= flag; \
+    else nd_config.flags &= ~flag; \
+}
+
 typedef struct nd_global_config_t {
     char *path_config;
-    char *path_content_match;
-    char *path_custom_match;
-    char *path_host_match;
+    char *path_sink_config;
     char *path_json;
     char *path_uuid;
     char *path_uuid_serial;
@@ -208,16 +209,13 @@ typedef struct nd_global_config_t {
     char *uuid_site;
     size_t max_backlog;
     uint32_t flags;
-    uint8_t digest_custom_match[SHA1_DIGEST_LENGTH];
-#ifndef _ND_LEAN_AND_MEAN
-    uint8_t digest_content_match[SHA1_DIGEST_LENGTH];
-    uint8_t digest_host_match[SHA1_DIGEST_LENGTH];
-#endif
+    uint8_t digest_sink_config[SHA1_DIGEST_LENGTH];
     unsigned max_tcp_pkts;
     unsigned max_udp_pkts;
     unsigned update_interval;
     unsigned upload_timeout;
     unsigned dns_cache_ttl;
+    FILE *h_flow;
 
     vector<pair<string, string> > socket_host;
     vector<string> socket_path;
@@ -225,7 +223,8 @@ typedef struct nd_global_config_t {
     vector<uint8_t *> privacy_filter_mac;
     nd_device_filter device_filters;
 #ifdef _ND_USE_PLUGINS
-    nd_plugin plugins;
+    map<string, string> services;
+    map<string, string> tasks;
 #endif
 } nd_global_config;
 
@@ -326,5 +325,34 @@ void nd_json_agent_hello(string &json_string);
 void nd_json_agent_status(string &json_string);
 void nd_json_protocols(string &json_string);
 
+class ndException : public runtime_error
+{
+public:
+    explicit ndException(
+        const string &where_arg, const string &what_arg) throw();
+    virtual ~ndException() throw();
+
+    virtual const char *what() const throw();
+
+    string where_arg;
+    string what_arg;
+    const char *message;
+};
+
+class ndSystemException : public runtime_error
+{
+public:
+    explicit ndSystemException(
+        const string &where_arg, const string &what_arg, int why_arg) throw();
+    virtual ~ndSystemException() throw();
+
+    virtual const char *what() const throw();
+
+    string where_arg;
+    string what_arg;
+    int why_arg;
+    const char *message;
+};
+
 #endif // _ND_H
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
