@@ -595,6 +595,9 @@ struct ndpi_flow_tcp_struct {
 
     /* NDPI_PROTOCOL_MEMCACHED */
     u_int8_t memcached_matches;
+
+    /* NDPI_PROTOCOL_NEST_LOG_SINK */
+    u_int8_t nest_log_sink_matches;
 }
 #ifndef WIN32
 __attribute__ ((__packed__))
@@ -909,7 +912,6 @@ struct ndpi_flow_struct {
 
     /* init parameter, internal used to set up timestamp,... */
     u_int16_t guessed_proto_id, guessed_host_proto_id;
-
     u_int8_t proto_id_already_guessed:1, host_already_guessed:1, init_finished:1, setup_packet_direction:1, packet_direction:1, check_extra_packets:1;
 
     /*
@@ -920,6 +922,7 @@ struct ndpi_flow_struct {
 
     u_int8_t max_extra_packets_to_check;
     u_int8_t num_extra_packets_checked;
+
     int (*extra_packets_func) (struct ndpi_detection_module_struct *, struct ndpi_flow_struct *flow);
 
     /*
