@@ -56,26 +56,41 @@ using namespace std;
 #include "netifyd.h"
 
 #include "nd-ndpi.h"
-#include "nd-util.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
 #include "nd-json.h"
 #include "nd-flow.h"
 #include "nd-thread.h"
-#include "nd-conntrack.h"
+#include "nd-util.h"
 
 // Enable Conntrack debug logging
-//#define _ND_LOG_CONNTRACK       1
-//#define _ND_LOG_CONNTRACK_INTV  15
+//#define _ND_DEBUG_CONNTRACK     1
+
+#include "nd-conntrack.h"
+
+extern nd_global_config nd_config;
+
+static time_t nd_ct_last_flow_purge_ttl = 0;
 
 static int nd_ct_event_callback(
     enum nf_conntrack_msg_type type, struct nf_conntrack *ct, void *data)
 {
     ndConntrackThread *thread = reinterpret_cast<ndConntrackThread *>(data);
+
     thread->ProcessConntrackEvent(type, ct);
 
-    return NFCT_CB_STOP;
+    time_t now = time(NULL);
+
+    if (now > nd_ct_last_flow_purge_ttl) {
+        thread->PurgeFlows();
+#ifdef _ND_DEBUG_CONNTRACK
+        thread->DumpStats();
+#endif
+        nd_ct_last_flow_purge_ttl = now + _ND_CT_FLOW_TTL + (_ND_CT_FLOW_TTL / 10);
+    }
+
+    return (thread->ShouldTerminate()) ? NFCT_CB_STOP : NFCT_CB_CONTINUE;
 }
 
 static int nd_ct_netlink_callback(const struct nlmsghdr *nlh, void *data)
@@ -87,7 +102,7 @@ static int nd_ct_netlink_callback(const struct nlmsghdr *nlh, void *data)
 
     if (nfct_nlmsg_parse(nlh, ct) == 0) {
         ndConntrackThread *thread = reinterpret_cast<ndConntrackThread *>(data);
-        thread->ProcessConntrackEvent(NFCT_T_UNKNOWN, ct);
+        thread->ProcessConntrackEvent(NFCT_T_NEW, ct);
     }
 
     nfct_destroy(ct);
@@ -97,7 +112,7 @@ static int nd_ct_netlink_callback(const struct nlmsghdr *nlh, void *data)
 
 ndConntrackThread::ndConntrackThread()
     : ndThread("nd-conntrack", -1),
-    ctfd(-1), cth(NULL), terminate(false), cb_registered(-1)
+    ctfd(-1), cth(NULL), cb_registered(-1)
 {
     cth = nfct_open(NFNL_SUBSYS_CTNETLINK, NFCT_ALL_CT_GROUPS);
     if (cth == NULL) {
@@ -110,6 +125,14 @@ ndConntrackThread::ndConntrackThread()
 
     ctfd = nfct_fd(cth);
 
+    int on = 1;
+
+    setsockopt(ctfd, SOL_NETLINK,
+        NETLINK_BROADCAST_SEND_ERROR, &on, sizeof(int));
+
+    setsockopt(ctfd, SOL_NETLINK,
+        NETLINK_NO_ENOBUFS, &on, sizeof(int));
+
     if ((cb_registered = nfct_callback_register(
         cth,
         NFCT_T_ALL,
@@ -126,8 +149,6 @@ ndConntrackThread::ndConntrackThread()
 
 ndConntrackThread::~ndConntrackThread()
 {
-    terminate = true;
-
     Join();
 
     if (cth != NULL) {
@@ -206,10 +227,10 @@ void *ndConntrackThread::Entry(void)
 {
     int rc;
     struct timeval tv;
-#ifdef _ND_LOG_CONNTRACK
-    time_t log_stats = time(NULL) + _ND_LOG_CONNTRACK_INTV;
-#endif
-    while (! terminate) {
+
+    nd_ct_last_flow_purge_ttl = time(NULL) + _ND_CT_FLOW_TTL;
+
+    while (! ShouldTerminate()) {
         fd_set fds_read;
 
         FD_ZERO(&fds_read);
@@ -231,13 +252,6 @@ void *ndConntrackThread::Entry(void)
                     __PRETTY_FUNCTION__, "nfct_catch", errno);
             }
         }
-#ifdef _ND_LOG_CONNTRACK
-        if (time(NULL) > log_stats) {
-            nd_debug_printf("%s: entries: ids: %lu, flows: %lu\n",
-                tag.c_str(), ct_id_map.size(), ct_flow_map.size());
-            log_stats = time(NULL) + _ND_LOG_CONNTRACK_INTV;
-        }
-#endif
     }
 
     nd_debug_printf("%s: Exit.\n", tag.c_str());
@@ -247,80 +261,107 @@ void *ndConntrackThread::Entry(void)
 void ndConntrackThread::ProcessConntrackEvent(
     enum nf_conntrack_msg_type type, struct nf_conntrack *ct)
 {
-    uint32_t id = nfct_get_attr_u32(ct, ATTR_ID);
     ndConntrackFlow *ct_flow = NULL;
-    nd_ct_id_map::iterator id_iter = ct_id_map.find(id);
+    nd_ct_id_map::iterator id_iter;
     nd_ct_flow_map::iterator flow_iter;
-#ifdef _ND_LOG_CONNTRACK
-    bool ct_exists = false, ct_new_or_update = false;
+
+    uint32_t id = nfct_get_attr_u32(ct, ATTR_ID);
+
+    Lock();
+
+    switch (type) {
+    case NFCT_T_NEW:
+
+#ifdef _ND_DEBUG_CONNTRACK
+        id_iter = ct_id_map.find(id);
+        if (id_iter != ct_id_map.end()) {
+            nd_debug_printf("%s: [N:%u] ID exists for new flow.\n",
+                tag.c_str(), id);
+        }
 #endif
 
-    if (id_iter == ct_id_map.end()) {
         try {
-            ct_flow = new ndConntrackFlow(ct);
+            ct_flow = new ndConntrackFlow(id, ct);
         }
         catch (ndConntrackFlowException &e) {
             nd_printf("%s: %s.\n", tag.c_str(), e.what());
-            return;
+            goto Unlock_ProcessConntrackEvent;
         }
 
         ct_id_map[id] = ct_flow->digest;
-        ct_flow_map[ct_flow->digest] = ct_flow;
-#ifdef _ND_LOG_CONNTRACK
-        ct_new_or_update = true;
-#endif
-    }
-    else {
-#ifdef _ND_LOG_CONNTRACK
-        ct_exists = true;
-#endif
-        flow_iter = ct_flow_map.find(id_iter->second);
 
-        if (type & NFCT_T_DESTROY) {
+        flow_iter = ct_flow_map.find(ct_flow->digest);
+        if (flow_iter != ct_flow_map.end()) {
+#ifdef _ND_DEBUG_CONNTRACK
+            nd_debug_printf("%s: [N:%u] Digest found in flow map.\n",
+                tag.c_str(), id);
+#endif
+            delete flow_iter->second;
+        }
+
+        ct_flow_map[ct_flow->digest] = ct_flow;
+        break;
+
+    case NFCT_T_UPDATE:
+
+        id_iter = ct_id_map.find(id);
+        if (id_iter == ct_id_map.end())
+            goto Unlock_ProcessConntrackEvent;
+
+        flow_iter = ct_flow_map.find(id_iter->second);
+        if (flow_iter == ct_flow_map.end()) {
+            nd_printf("%s: [U:%u] Digest not found in flow map.\n",
+                tag.c_str(), id);
+            ct_id_map.erase(id_iter);
+            goto Unlock_ProcessConntrackEvent;
+        }
+
+        ct_flow = flow_iter->second;
+
+        ct_flow->Update(ct);
+
+        if (ct_flow->digest != id_iter->second) {
+#ifdef _ND_DEBUG_CONNTRACK
+            nd_debug_printf("%s: [U:%u] Flow hash updated.\n",
+                tag.c_str(), id);
+#endif
+            ct_flow_map.erase(flow_iter);
+
+            ct_flow_map[ct_flow->digest] = ct_flow;
+            ct_id_map[id] = ct_flow->digest;
+        }
+        break;
+
+    case NFCT_T_DESTROY:
+
+        id_iter = ct_id_map.find(id);
+        if (id_iter != ct_id_map.end()) {
+
+            flow_iter = ct_flow_map.find(id_iter->second);
             if (flow_iter != ct_flow_map.end()) {
                 delete flow_iter->second;
                 ct_flow_map.erase(flow_iter);
             }
+
             ct_id_map.erase(id_iter);
         }
-        else {
-            if (flow_iter == ct_flow_map.end()) {
-                nd_debug_printf("%s: [%u] Connection tracking flow not found!\n",
-                    tag.c_str(), id);
-                ct_id_map.erase(id_iter);
-                return;
-            }
+        break;
 
-            ct_flow = flow_iter->second;
-            ct_flow->Update(ct);
-#ifdef _ND_LOG_CONNTRACK
-            ct_new_or_update = true;
-#endif
-            if (ct_flow->digest != id_iter->second) {
-                nd_debug_printf("%s: [%u] Connection tracking flow hash changed!\n",
-                    tag.c_str(), id);
-                ct_flow_map.erase(flow_iter);
-                ct_flow_map[ct_flow->digest] = ct_flow;
-                ct_id_map[id] = ct_flow->digest;
-            }
-        }
+    default:
+        nd_printf("%s: Unhandled connection tracking message type: 0x%02x\n",
+            tag.c_str(), type);
     }
-#ifdef _ND_LOG_CONNTRACK
-    if (nd_debug) {
+#ifdef _ND_DEBUG_CONNTRACK
+    if (ND_DEBUG) {
+#if 0
         char buffer[1024];
         nfct_snprintf(buffer, sizeof(buffer), ct, type, NFCT_O_PLAIN, NFCT_OF_TIME);
-
-        if (! ct_exists && ct_new_or_update)
-            nd_debug_printf("%s: [%u] %s\n", tag.c_str(), id, buffer);
-//        if (! ct_new_or_update && ! ct_exists)
-//            nd_printf("%s: [%u] %s\n", tag.c_str(), id, buffer);
-//        nd_printf("%s: [%s %u] %s\n", tag.c_str(),
-//            (ct_new_or_update && ! ct_exists) ?
-//                "INSERT" : (ct_new_or_update && ct_exists) ?
-//                "UPDATE" : (ct_exists && ! ct_new_or_update) ?
-//                "ERASE" : "UNKNOWN", id, buffer);
+        nd_debug_printf("%s: %02x [%u] %s\n", tag.c_str(), type, id, buffer);
+#endif
     }
 #endif
+Unlock_ProcessConntrackEvent:
+    Unlock();
 }
 
 void ndConntrackThread::PrintFlow(
@@ -582,9 +623,41 @@ void ndConntrackThread::ClassifyFlow(ndFlow *flow)
     Unlock();
 }
 
-ndConntrackFlow::ndConntrackFlow(struct nf_conntrack *ct)
-    : l3_proto(0), l4_proto(0)
+void ndConntrackThread::PurgeFlows(void)
 {
+    Lock();
+
+    for (nd_ct_flow_map::iterator i = ct_flow_map.begin();
+        i != ct_flow_map.end(); ) {
+
+        if (i->second->HasExpired()) {
+            ct_id_map.erase(i->second->GetId());
+            delete i->second;
+            i = ct_flow_map.erase(i);
+        }
+        else i++;
+    }
+
+    Unlock();
+}
+
+#ifdef _ND_DEBUG_CONNTRACK
+void ndConntrackThread::DumpStats(void)
+{
+    Lock();
+
+    nd_debug_printf("%s: entries: ids: %lu, flows: %lu\n",
+        tag.c_str(), ct_id_map.size(), ct_flow_map.size());
+
+    Unlock();
+}
+#endif
+
+ndConntrackFlow::ndConntrackFlow(uint32_t id, struct nf_conntrack *ct)
+    : id(id), created_at(0), l3_proto(0), l4_proto(0)
+{
+    created_at = time(NULL);
+
     orig_port[ndCT_DIR_SRC] = 0;
     orig_port[ndCT_DIR_DST] = 0;
     repl_port[ndCT_DIR_SRC] = 0;
