@@ -38,6 +38,7 @@ typedef bool atomic_bool;
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
@@ -123,7 +124,6 @@ using namespace std;
 #include "netifyd.h"
 
 #include "nd-ndpi.h"
-#include "nd-util.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
@@ -133,8 +133,9 @@ using namespace std;
 #ifdef _ND_USE_CONNTRACK
 #include "nd-conntrack.h"
 #endif
-#include "nd-detection.h"
 #include "nd-socket.h"
+#include "nd-util.h"
+#include "nd-detection.h"
 
 // Enable to log discarded packets
 //#define _ND_LOG_PKT_DISCARD     1
@@ -314,8 +315,6 @@ void *ndDetectionThread::Entry(void)
     nd_debug_printf("%s: capture ended on CPU: %lu\n",
         tag.c_str(), cpu >= 0 ? cpu : 0);
 
-    terminated = true;
-
     return NULL;
 }
 
@@ -1211,7 +1210,7 @@ void ndDetectionThread::ProcessPacket(void)
             }
         }
 
-        if (ND_DEBUG)
+        if (ND_DEBUG || nd_config.h_flow != stderr)
             new_flow->print(tag.c_str(), ndpi);
 
         if (thread_socket) {
