@@ -25,10 +25,13 @@
 #include <unordered_map>
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
+
 #include <errno.h>
 
+#include <arpa/inet.h>
+
 #ifdef _ND_USE_NETLINK
 #include <linux/netlink.h>
 #endif
@@ -39,11 +42,11 @@ using namespace std;
 #include "netifyd.h"
 
 #include "nd-ndpi.h"
-#include "nd-util.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
 #include "nd-json.h"
+#include "nd-util.h"
 #include "nd-flow.h"
 
 extern nd_global_config nd_config;
@@ -261,8 +264,7 @@ void ndFlow::print(const char *tag, struct ndpi_detection_module_struct *ndpi)
     string digest;
     nd_sha1_to_string((const uint8_t *)bt.info_hash, digest);
 
-    //"%s: [%c%c%c%c%c%c] %s %s:%hu <+> %s:%hu%s%s%s%s%s%s%s%s%s\n",
-    nd_debug_printf(
+    nd_flow_printf(
         "%s: [%c%c%c%c%c%c] %s %s:%hu %c%c%c %s:%hu%s%s%s%s%s%s%s%s%s\n",
         tag,
         (internal) ? 'i' : 'e',
@@ -439,6 +441,27 @@ json_object *ndFlow::json_encode(const string &device,
         _upper_bytes = "other_bytes";
         _upper_packets = "other_packets";
     }
+#if 0
+    // TODO: Further investigation required!
+    // This appears to catch corrupted IPv6 headers.
+    // Spend some time to figure out if there are any
+    // possible over-matches for different methods of
+    // deployment (gateway/port mirror modes).
+    else if (lower_type == ndNETLINK_ATYPE_PRIVATE &&
+        upper_type == ndNETLINK_ATYPE_PRIVATE) {
+        other_type = "local";
+        _lower_mac = "local_mac";
+        _lower_ip = "local_ip";
+        _lower_port = "local_port";
+        _lower_bytes = "local_bytes";
+        _lower_packets = "local_packets";
+        _upper_mac = "other_mac";
+        _upper_ip = "other_ip";
+        _upper_port = "other_port";
+        _upper_bytes = "other_bytes";
+        _upper_packets = "other_packets";
+    }
+#endif
     else if (lower_type == ndNETLINK_ATYPE_PRIVATE &&
         upper_type == ndNETLINK_ATYPE_LOCALIP) {
         other_type = "remote";
@@ -513,7 +536,7 @@ json_object *ndFlow::json_encode(const string &device,
     if (ND_DEBUG && other_type == "unknown") {
         ndNetlink::PrintType(lower_ip, lower_type);
         ndNetlink::PrintType(upper_ip, upper_type);
-        exit(1);
+        //exit(1);
     }
 #endif
 #endif
