@@ -78,7 +78,6 @@ using namespace std;
 #include "netifyd.h"
 
 #include "nd-ndpi.h"
-#include "nd-util.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
@@ -88,6 +87,7 @@ using namespace std;
 #ifdef _ND_USE_CONNTRACK
 #include "nd-conntrack.h"
 #endif
+#include "nd-util.h"
 #include "nd-socket.h"
 
 #define _ND_SOCKET_PROC_NET_UNIX    "/proc/net/unix"
@@ -653,22 +653,23 @@ void ndSocketThread::QueueWrite(const string &data)
 void ndSocketThread::ClientAccept(ndSocketServerMap::iterator &si)
 {
     ndSocket *client = NULL;
-    ndSocketBuffer *buffer = NULL;
 
     try {
-        buffer = new ndSocketBuffer();
-        if (buffer == NULL)
-            throw ndSocketThreadException(__PRETTY_FUNCTION__, "new", ENOMEM);
-
         client = si->second->Accept();
     } catch (ndSocketGetAddrInfoException &e) {
-        if (client) delete client;
+        if (client != NULL) delete client;
         throw;
     } catch (ndSocketSystemException &e) {
-        if (client) delete client;
+        if (client != NULL) delete client;
         throw;
     }
 
+    ndSocketBuffer *buffer = new ndSocketBuffer();
+    if (buffer == NULL) {
+        delete client;
+        throw ndSocketThreadException(__PRETTY_FUNCTION__, "new", ENOMEM);
+    }
+
     buffers[client->GetDescriptor()] = buffer;
     clients[client->GetDescriptor()] = client;
 
@@ -815,8 +816,6 @@ void *ndSocketThread::Entry(void)
         }
     }
 
-    terminated = true;
-
     return NULL;
 }
 
