@@ -46,6 +46,7 @@ typedef bool atomic_bool;
 #include <unistd.h>
 #include <locale.h>
 #include <syslog.h>
+#include <fcntl.h>
 
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
@@ -74,7 +75,6 @@ using namespace std;
 #include "netifyd.h"
 
 #include "nd-ndpi.h"
-#include "nd-util.h"
 #ifdef _ND_USE_INOTIFY
 #include "nd-inotify.h"
 #endif
@@ -89,17 +89,16 @@ using namespace std;
 #endif
 #include "nd-detection.h"
 #include "nd-socket.h"
-#include "nd-upload.h"
+#include "nd-sink.h"
+#include "nd-base64.h"
 #ifdef _ND_USE_PLUGINS
 #include "nd-plugin.h"
 #endif
+#include "nd-util.h"
 
 #define ND_SIG_UPDATE       SIGRTMIN
 #define ND_STR_ETHALEN     (ETH_ALEN * 2 + ETH_ALEN - 1)
 
-nd_global_config nd_config;
-pthread_mutex_t *nd_printf_mutex = NULL;
-
 static nd_ifaces ifaces;
 static nd_devices devices;
 nd_device_ethers device_ethers;
@@ -109,8 +108,12 @@ static nd_threads threads;
 static nd_agent_stats nda_stats;
 static nd_packet_stats pkt_totals;
 static ostringstream *nd_stats_os = NULL;
-static ndUploadThread *thread_upload = NULL;
+static ndSinkThread *thread_sink = NULL;
 static ndSocketThread *thread_socket = NULL;
+#ifdef _ND_USE_PLUGINS
+static nd_plugins plugin_services;
+static nd_plugins plugin_tasks;
+#endif
 static char *nd_conf_filename = NULL;
 #ifdef _ND_USE_CONNTRACK
 static ndConntrackThread *thread_conntrack = NULL;
@@ -124,9 +127,12 @@ static ndNetlink *netlink = NULL;
 nd_device_netlink device_netlink;
 #endif
 static nd_device_filter device_filters;
-
+static bool nd_detection_stopped_by_signal = false;
 static time_t nd_ethers_mtime = 0;
 
+nd_global_config nd_config;
+pthread_mutex_t *nd_printf_mutex = NULL;
+
 void nd_dns_cache::insert(sa_family_t af, const uint8_t *addr, const string &hostname)
 {
     sha1 ctx;
@@ -334,11 +340,11 @@ static void nd_config_init(void)
 {
     nd_conf_filename = strdup(ND_CONF_FILE_NAME);
 
+    nd_config.h_flow = stderr;
+
     nd_config.path_config = NULL;
-    nd_config.path_content_match = NULL;
-    nd_config.path_custom_match = NULL;
-    nd_config.path_host_match = NULL;
     nd_config.path_json = NULL;
+    nd_config.path_sink_config = strdup(ND_CONF_SINK_PATH);
     nd_config.path_uuid = NULL;
     nd_config.path_uuid_serial = NULL;
     nd_config.path_uuid_site = NULL;
@@ -348,30 +354,28 @@ static void nd_config_init(void)
     nd_config.uuid_site = NULL;
 
     nd_config.max_backlog = ND_MAX_BACKLOG_KB * 1024;
-#if defined(_ND_USE_CONNTRACK) && defined(_ND_USE_NETLINK)
-    nd_config.flags = ndGF_USE_CONNTRACK | ndGF_USE_NETLINK;
-#elif defined(_ND_USE_CONNTRACK)
-    nd_config.flags = ndGF_USE_CONNTRACK;
-#elif defined(_ND_USE_NETLINK)
-    nd_config.flags = ndGF_USE_NETLINK;
+
+    nd_config.flags |= ndGF_SSL_VERIFY;
+#ifdef _ND_USE_CONNTRACK
+    nd_config.flags |= ndGF_USE_CONNTRACK;
 #endif
-    nd_config.path_custom_match = strdup(ND_CONF_CUSTOM_MATCH);
-    memset(nd_config.digest_custom_match, 0, SHA1_DIGEST_LENGTH);
-#ifndef _ND_LEAN_AND_MEAN
-    nd_config.path_content_match = strdup(ND_CONF_CONTENT_MATCH);
-    nd_config.path_host_match = strdup(ND_CONF_HOST_MATCH);
-    memset(nd_config.digest_content_match, 0, SHA1_DIGEST_LENGTH);
-    memset(nd_config.digest_host_match, 0, SHA1_DIGEST_LENGTH);
+#ifdef _ND_USE_NETLINK
+    nd_config.flags |= ndGF_USE_NETLINK;
 #endif
+
     nd_config.max_tcp_pkts = ND_MAX_TCP_PKTS;
     nd_config.max_udp_pkts = ND_MAX_UDP_PKTS;
     nd_config.update_interval = ND_STATS_INTERVAL;
     nd_config.upload_timeout = ND_UPLOAD_TIMEOUT;
     nd_config.dns_cache_ttl = ND_IDLE_DNS_CACHE_TTL;
+
+    memset(nd_config.digest_sink_config, 0, SHA1_DIGEST_LENGTH);
 }
 
 static int nd_config_load(void)
 {
+    typedef map<string, string> nd_config_section;
+
     if (nd_conf_filename == NULL) {
         fprintf(stderr, "Configuration file not defined.\n");
         return -1;
@@ -398,6 +402,10 @@ static int nd_config_load(void)
             nd_config.uuid = strdup(uuid.c_str());
     }
 
+    // Netify section
+    nd_config_section netifyd_section;
+    reader.GetSection("netifyd", netifyd_section);
+
     string path_uuid = reader.Get(
         "netifyd", "path_uuid", ND_AGENT_UUID_PATH);
     nd_config.path_uuid = strdup(path_uuid.c_str());
@@ -428,32 +436,25 @@ static int nd_config_load(void)
     nd_config.upload_timeout = (unsigned)reader.GetInteger(
         "netifyd", "upload_timeout", ND_UPLOAD_TIMEOUT);
 
-    nd_config.flags |= (reader.GetBoolean(
-        "netifyd", "json_save", false)) ? ndGF_JSON_SAVE : 0;
-
-    nd_config.flags |= (reader.GetBoolean(
-        "dns_cache", "enable", true)) ? ndGF_USE_DNS_CACHE : 0;
-
-    nd_config.flags |= (reader.GetBoolean(
-        "dns_cache", "save", true)) ? ndGF_DNS_CACHE_SAVE : 0;
-
-    nd_config.dns_cache_ttl = (unsigned)reader.GetInteger(
-        "dns_cache", "cache_ttl", ND_IDLE_DNS_CACHE_TTL);
+    ND_GF_SET_FLAG(ndGF_JSON_SAVE,
+        reader.GetBoolean("netifyd", "json_save", false));
 
     nd_config.max_backlog = reader.GetInteger(
         "netifyd", "max_backlog_kb", ND_MAX_BACKLOG_KB) * 1024;
 
-    nd_config.flags |= (reader.GetBoolean(
-        "netifyd", "enable_sink", false)) ? ndGF_USE_SINK : 0;
+    ND_GF_SET_FLAG(ndGF_USE_SINK,
+        reader.GetBoolean("netifyd", "enable_sink", false));
 
-    nd_config.flags |= (reader.GetBoolean(
-        "netifyd", "enable_netify_sink", false)) ? ndGF_USE_SINK : 0;
+    if (netifyd_section.find("ssl_verify") != netifyd_section.end()) {
+        ND_GF_SET_FLAG(ndGF_SSL_VERIFY,
+            reader.GetBoolean("netifyd", "ssl_verify", true));
+    } else if (netifyd_section.find("ssl_verify_peer") != netifyd_section.end()) {
+        ND_GF_SET_FLAG(ndGF_SSL_VERIFY,
+            reader.GetBoolean("netifyd", "ssl_verify_peer", true));
+    }
 
-    nd_config.flags |= (reader.GetBoolean(
-        "netifyd", "ssl_verify_peer", true)) ? ndGF_SSL_VERIFY_PEER : 0;
-
-    nd_config.flags |= (reader.GetBoolean(
-        "netifyd", "ssl_use_tlsv1", false)) ? ndGF_SSL_USE_TLSv1 : 0;
+    ND_GF_SET_FLAG(ndGF_SSL_USE_TLSv1,
+        reader.GetBoolean("netifyd", "ssl_use_tlsv1", false));
 
     nd_config.max_tcp_pkts = (unsigned)reader.GetInteger(
         "netifyd", "max_tcp_pkts", ND_MAX_TCP_PKTS);
@@ -461,6 +462,17 @@ static int nd_config_load(void)
     nd_config.max_udp_pkts = (unsigned)reader.GetInteger(
         "netifyd", "max_udp_pkts", ND_MAX_UDP_PKTS);
 
+    // DNS Cache section
+    ND_GF_SET_FLAG(ndGF_USE_DNS_CACHE,
+        reader.GetBoolean("dns_cache", "enable", true));
+
+    ND_GF_SET_FLAG(ndGF_DNS_CACHE_SAVE,
+        reader.GetBoolean("dns_cache", "save", true));
+
+    nd_config.dns_cache_ttl = (unsigned)reader.GetInteger(
+        "dns_cache", "cache_ttl", ND_IDLE_DNS_CACHE_TTL);
+
+    // Socket section
     for (int i = 0; ; i++) {
         ostringstream os;
         os << "listen_address[" << i << "]";
@@ -487,6 +499,7 @@ static int nd_config_load(void)
         break;
     }
 
+    // Privacy filter section
     for (int i = 0; ; i++) {
         ostringstream os;
         os << "mac[" << i << "]";
@@ -537,21 +550,13 @@ static int nd_config_load(void)
         freeaddrinfo(result);
     }
 #ifdef _ND_USE_INOTIFY
+    // Watches section
     reader.GetSection("watches", inotify_watches);
 #endif
 #ifdef _ND_USE_PLUGINS
-    map<string, string> plugins;
-    reader.GetSection("plugins", plugins);
-
-    for (map<string, string>::const_iterator i = plugins.begin();
-        i != plugins.end(); i++) {
-        try {
-            nd_config.plugins[i->first] = new ndPluginLoader(i->second, i->first);
-        } catch (ndPluginException &e) {
-            fprintf(stderr, "Error loading plugin: %s %s: %s\n",
-                i->first.c_str(), i->second.c_str(), e.what());
-        }
-    }
+    // Plugins section
+    reader.GetSection("services", nd_config.services);
+    reader.GetSection("tasks", nd_config.tasks);
 #endif
     return 0;
 }
@@ -562,7 +567,7 @@ static int nd_config_load(void)
 static int nd_config_set_option(int option)
 {
     ostringstream os;
-    os << "sh -c \"source " << ND_LIBEXECDIR << "/functions.sh && config_";
+    os << "sh -c \"source " << ND_DATADIR << "/functions.sh && config_";
 
     switch (option) {
     case _ND_LO_ENABLE_SINK:
@@ -610,6 +615,8 @@ static int nd_config_set_option(int option)
 
 static int nd_start_detection_threads(void)
 {
+    if (threads.size() > 0) return 1;
+
     ndpi_global_init();
 
     for (nd_ifaces::iterator i = ifaces.begin();
@@ -652,7 +659,6 @@ static int nd_start_detection_threads(void)
                 netlink,
 #endif
                 thread_socket,
-//                (i->first) ? thread_socket : NULL,
 #ifdef _ND_USE_CONNTRACK
                 (i->first || ! ND_USE_CONNTRACK) ?
                     NULL : thread_conntrack,
@@ -679,6 +685,8 @@ static int nd_start_detection_threads(void)
 
 static void nd_stop_detection_threads(void)
 {
+    if (threads.size() == 0) return;
+
     for (nd_ifaces::iterator i = ifaces.begin();
         i != ifaces.end(); i++) {
         threads[(*i).second]->Terminate();
@@ -706,6 +714,8 @@ static void nd_stop_detection_threads(void)
 
 static void nd_reap_detection_threads(void)
 {
+    if (threads.size() == 0) return;
+
     nd_threads::iterator thread_iter;
     nd_flows::iterator flow_iter;
     nd_stats::iterator stat_iter;
@@ -749,6 +759,224 @@ static void nd_reap_detection_threads(void)
         threads.erase(thread_iter);
         iface_iter = ifaces.erase(iface_iter);
     }
+
+    if (threads.size() == 0) ndpi_global_destroy();
+}
+
+#ifdef _ND_USE_PLUGINS
+
+static int nd_start_services(void)
+{
+    for (map<string, string>::const_iterator i = nd_config.services.begin();
+        i != nd_config.services.end(); i++) {
+        try {
+            plugin_services[i->first] = new ndPluginLoader(i->second, i->first);
+            plugin_services[i->first]->GetPlugin()->Create();
+        }
+        catch (ndPluginException &e) {
+            nd_printf("Error loading service plugin: %s\n", e.what());
+            return 1;
+        }
+        catch (ndThreadException &e) {
+            nd_printf("Error starting service plugin: %s %s: %s\n",
+                i->first.c_str(), i->second.c_str(), e.what());
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
+static void nd_stop_services(void)
+{
+    for (nd_plugins::iterator i = plugin_services.begin();
+        i != plugin_services.end(); i++) {
+
+        ndPluginService *service = reinterpret_cast<ndPluginService *>(
+            i->second->GetPlugin()
+        );
+        service->Terminate();
+        delete service;
+
+        delete i->second;
+    }
+
+    plugin_services.clear();
+}
+
+static int nd_dispatch_service_param(
+    const string &name, const string &uuid_dispatch, const ndJsonPluginParams &params)
+{
+    int rc = 0;
+
+    nd_plugins::iterator plugin_iter = plugin_services.find(name);
+
+    if (plugin_iter == plugin_services.end()) {
+        nd_printf("Unable to dispatch parameters; service not found: %s\n",
+            name.c_str());
+        rc = -1;
+    }
+    else {
+        ndPluginService *service = reinterpret_cast<ndPluginService *>(
+            plugin_iter->second->GetPlugin()
+        );
+
+        service->SetParams(uuid_dispatch, params);
+    }
+
+    return rc;
+}
+
+static int nd_start_task(
+    const string &name, const string &uuid_dispatch, const ndJsonPluginParams &params)
+{
+    map<string, string>::const_iterator task_iter = nd_config.tasks.find(name);
+
+    if (task_iter == nd_config.tasks.end()) {
+        nd_printf("Unable to initialize plugin; task not found: %s\n",
+            name.c_str());
+        return -1;
+    }
+
+    nd_plugins::iterator plugin_iter = plugin_tasks.find(uuid_dispatch);
+
+    if (plugin_iter != plugin_tasks.end()) {
+        nd_printf("Unable to initialize plugin; task exists: %s: %s\n",
+            name.c_str(), uuid_dispatch.c_str());
+        return -1;
+    }
+
+    try {
+        ndPluginLoader *plugin = new ndPluginLoader(
+            task_iter->second, task_iter->first
+        );
+
+        ndPluginTask *task = reinterpret_cast<ndPluginTask *>(
+            plugin->GetPlugin()
+        );
+
+        task->SetParams(uuid_dispatch, params);
+        task->Create();
+
+        plugin_tasks[uuid_dispatch] = plugin;
+    }
+    catch (ndPluginException &e) {
+        nd_printf("Error loading task plugin: %s\n", e.what());
+        return -1;
+    }
+    catch (ndThreadException &e) {
+        nd_printf("Error starting task plugin: %s %s: %s\n",
+            task_iter->first.c_str(), task_iter->second.c_str(), e.what());
+        return -1;
+    }
+
+    return 0;
+}
+
+static void nd_stop_tasks(void)
+{
+    for (nd_plugins::iterator i = plugin_tasks.begin();
+        i != plugin_tasks.end(); i++) {
+
+        ndPluginTask *task = reinterpret_cast<ndPluginTask *>(
+            i->second->GetPlugin()
+        );
+        task->Terminate();
+        delete task;
+    }
+}
+
+static void nd_reap_tasks(void)
+{
+    for (nd_plugins::iterator i = plugin_tasks.begin();
+        i != plugin_tasks.end(); i++) {
+        if (! i->second->GetPlugin()->HasTerminated()) continue;
+
+        nd_debug_printf("Reaping task plugin: %s: %s\n",
+            i->second->GetPlugin()->GetTag().c_str(),
+            i->first.c_str());
+
+        delete i->second->GetPlugin();
+        delete i->second;
+
+        plugin_tasks.erase(i);
+    }
+}
+
+#endif // _USE_ND_PLUGINS
+
+static int nd_sink_process_responses(void)
+{
+    int count = 0;
+    bool reloaded = false;
+
+    while (true) {
+        ndJsonResponse *response = thread_sink->PopResponse();
+
+        if (response == NULL) break;
+
+        count++;
+
+        for (ndJsonData::const_iterator i = response->data.begin();
+            i != response->data.end(); i++) {
+
+            if (! reloaded && i->first == ND_CONF_SINK_BASE) {
+
+                if (! nd_detection_stopped_by_signal) {
+                    nd_stop_detection_threads();
+                    if (nd_start_detection_threads() < 0) return -1;
+                }
+
+                if (thread_socket) {
+                    string json;
+                    nd_json_protocols(json);
+                    thread_socket->QueueWrite(json);
+                }
+
+                reloaded = true;
+            }
+        }
+
+#ifdef _ND_USE_PLUGINS
+        for (ndJsonPluginRequest::const_iterator
+            i = response->plugin_request_service_param.begin();
+            i != response->plugin_request_service_param.end(); i++) {
+
+            ndJsonPluginDispatch::const_iterator iter_params;
+            iter_params = response->plugin_params.find(i->first);
+
+            if (iter_params != response->plugin_params.end()) {
+                const ndJsonPluginParams &params(iter_params->second);
+                nd_dispatch_service_param(i->second, i->first, params);
+            }
+            else {
+                const ndJsonPluginParams params;
+                nd_dispatch_service_param(i->second, i->first, params);
+            }
+        }
+
+        for (ndJsonPluginRequest::const_iterator
+            i = response->plugin_request_task_exec.begin();
+            i != response->plugin_request_task_exec.end(); i++) {
+
+            ndJsonPluginDispatch::const_iterator iter_params;
+            iter_params = response->plugin_params.find(i->first);
+
+            if (iter_params != response->plugin_params.end()) {
+                const ndJsonPluginParams &params(iter_params->second);
+                nd_start_task(i->second, i->first, params);
+            }
+            else {
+                const ndJsonPluginParams params;
+                nd_start_task(i->second, i->first, params);
+            }
+        }
+#endif
+
+        delete response;
+    }
+
+    return count;
 }
 
 void nd_json_agent_hello(string &json_string)
@@ -798,9 +1026,7 @@ void nd_json_protocols(string &json_string)
     json.AddObject(NULL, "type", "protocols");
     json_object *jarray = json.CreateArray(NULL, "protocols");
 
-    uint32_t custom_proto_base;
-    struct ndpi_detection_module_struct *ndpi;
-    ndpi = nd_ndpi_init("netifyd", custom_proto_base);
+    struct ndpi_detection_module_struct *ndpi = ndpi_get_parent();
 
     for (unsigned i = 0; i < (unsigned)ndpi->ndpi_num_supported_protocols; i++) {
         json_object *json_proto = json.CreateObject();
@@ -809,8 +1035,6 @@ void nd_json_protocols(string &json_string)
         json.PushObject(jarray, json_proto);
     }
 
-    ndpi_free(ndpi);
-
     json.ToString(json_string, false);
     json_string.append("\n");
 
@@ -965,33 +1189,167 @@ static void nd_json_add_flows(
 }
 
 static void nd_json_add_file(
-    json_object *parent, const string &type, const string &filename)
+    json_object *parent, const string &tag, const string &filename)
 {
-    char *c, *p, buffer[ND_FILE_BUFSIZ];
-    FILE *fh = fopen(filename.c_str(), "r");
+    string digest;
+    uint8_t _digest[SHA1_DIGEST_LENGTH];
 
-    if (fh == NULL) {
+    if (nd_sha1_file(filename.c_str(), _digest) < 0) return;
+
+    nd_sha1_to_string(_digest, digest);
+
+    uint8_t buffer[ND_JSON_DATA_CHUNKSIZ];
+    int fd = open(filename.c_str(), O_RDONLY);
+
+    if (fd < 0) {
         nd_printf("Error opening file for upload: %s: %s\n",
             filename.c_str(), strerror(errno));
         return;
     }
 
-    ndJson json(parent);
-    json_object *json_lines = json.CreateArray(NULL, type.c_str());
-
-    p = buffer;
-    while (fgets(buffer, ND_FILE_BUFSIZ, fh) != NULL) {
-        while (*p == ' ' || *p == '\t') p++;
-        if (*p == '#' || *p == '\n' || *p == '\r') continue;
-        c = (char *)memchr((void *)p, '\n', ND_FILE_BUFSIZ - (p - buffer));
-        if (c != NULL) *c = '\0';
-
-        json.PushObject(json_lines, p);
+    struct stat file_stat;
+    if (fstat(fd, &file_stat) != 0) {
+        nd_printf("Error reading stats for upload file: %s: %s\n",
+            filename.c_str(), strerror(errno));
+        close(fd);
+        return;
     }
 
-    fclose(fh);
+    ndJson json(parent);
+    json_object *json_tag = json.CreateObject(NULL, tag.c_str());
+
+    json.AddObject(json_tag, "digest", digest);
+    // XXX: Cast down to 32-bit unsigned:
+    json.AddObject(json_tag, "size", (uint32_t)file_stat.st_size);
+
+    json_object *json_chunks = json.CreateArray(json_tag, "chunks");
+
+    size_t bytes;
+
+    do {
+        if ((bytes = read(fd, buffer, ND_JSON_DATA_CHUNKSIZ)) > 0)
+            json.PushObject(json_chunks, base64_encode(buffer, bytes));
+    }
+    while (bytes > 0);
+
+    close(fd);
 }
 
+static void nd_json_add_data(
+    json_object *parent, const string &tag, const string &data)
+{
+    sha1 ctx;
+    string digest;
+    uint8_t _digest[SHA1_DIGEST_LENGTH];
+
+    sha1_init(&ctx);
+
+    ndJson json(parent);
+    json_object *json_tag = json.CreateObject(NULL, tag.c_str());
+
+    json_object *json_chunks = json.CreateArray(json_tag, "chunks");
+
+    // XXX: Cast down to 32-bit unsigned:
+    json.AddObject(json_tag, "size", (uint32_t)data.size());
+
+    size_t offset = 0;
+
+    do {
+        const string chunk = data.substr(offset, ND_JSON_DATA_CHUNKSIZ);
+
+        if (! chunk.size()) break;
+
+        sha1_write(&ctx, chunk.c_str(), chunk.size());
+        json.PushObject(json_chunks,
+            base64_encode((const unsigned char *)chunk.c_str(), chunk.size()));
+
+        if (chunk.size() != ND_JSON_DATA_CHUNKSIZ) break;
+
+        offset += ND_JSON_DATA_CHUNKSIZ;
+    }
+    while (offset < data.size());
+
+    digest.assign((const char *)sha1_result(&ctx, _digest), SHA1_DIGEST_LENGTH);
+    nd_sha1_to_string(_digest, digest);
+    json.AddObject(json_tag, "digest", digest);
+}
+
+#ifdef _ND_USE_PLUGINS
+static void nd_json_add_plugin_replies(json_object *json_plugin_service_replies,
+    json_object *json_plugin_task_replies, json_object *json_data)
+{
+    vector<ndPlugin *> plugins;
+    ndJson json_services(json_plugin_service_replies);
+    ndJson json_tasks(json_plugin_task_replies);
+
+    for (nd_plugins::const_iterator i = plugin_services.begin();
+        i != plugin_services.end(); i++)
+        plugins.push_back(i->second->GetPlugin());
+    for (nd_plugins::const_iterator i = plugin_tasks.begin();
+        i != plugin_tasks.end(); i++)
+        plugins.push_back(i->second->GetPlugin());
+
+    for (vector<ndPlugin *>::const_iterator i = plugins.begin();
+        i != plugins.end(); i++) {
+
+        ndJson *parent = NULL;
+
+        switch ((*i)->GetType()) {
+
+        case ndPlugin::TYPE_SERVICE:
+            parent = &json_services;
+            break;
+        case ndPlugin::TYPE_TASK:
+            parent = &json_tasks;
+            break;
+
+        default:
+            nd_debug_printf("%s: Unsupported plugin type: %d\n",
+                __PRETTY_FUNCTION__, (*i)->GetType());
+        }
+
+        if (parent == NULL) continue;
+
+        ndPluginFiles files, data;
+        ndPluginReplies replies;
+        (*i)->GetReplies(files, data, replies);
+
+        nd_debug_printf("%s: files: %ld, data: %ld, replies: %ld\n",
+            __PRETTY_FUNCTION__, files.size(), data.size(), replies.size());
+
+        if (! replies.size()) continue;
+
+        for (ndPluginReplies::const_iterator iter_reply = replies.begin();
+            iter_reply != replies.end(); iter_reply++) {
+
+            json_object *jarray = parent->CreateArray(NULL, iter_reply->first.c_str());
+
+            for (ndJsonPluginReplies::const_iterator iter_params = iter_reply->second.begin();
+                iter_params != iter_reply->second.end(); iter_params++) {
+
+                json_object *json_reply = parent->CreateObject();
+
+                parent->AddObject(json_reply, iter_params->first,
+                    base64_encode((const unsigned char *)iter_params->second.c_str(),
+                        iter_params->second.size())
+                );
+                parent->PushObject(jarray, json_reply);
+            }
+        }
+
+        for (ndPluginFiles::const_iterator iter_file = files.begin();
+            iter_file != files.end(); iter_file++) {
+            nd_json_add_file(json_data, iter_file->first, iter_file->second);
+        }
+
+        for (ndPluginFiles::const_iterator iter_file = data.begin();
+            iter_file != data.end(); iter_file++) {
+            nd_json_add_data(json_data, iter_file->first, iter_file->second);
+        }
+    }
+}
+#endif
+
 static void nd_print_stats(void)
 {
 #ifndef _ND_LEAN_AND_MEAN
@@ -1069,6 +1427,7 @@ static void nd_print_stats(void)
 #endif // _ND_LEAN_AND_MEAN
 }
 
+#ifndef _ND_LEAN_AND_MEAN
 static void nd_load_ethers(void)
 {
     char buffer[1024 + ND_STR_ETHALEN + 17];
@@ -1126,10 +1485,10 @@ static void nd_load_ethers(void)
     nd_debug_printf("Loaded %lu entries from: %s\n",
         device_ethers.size(), ND_ETHERS_FILE_NAME);
 }
+#endif
 
 static void nd_dump_stats(void)
 {
-    string digest;
 #if defined(_ND_USE_LIBTCMALLOC) && defined(HAVE_GPERFTOOLS_MALLOC_EXTENSION_H)
     size_t tcm_alloc_bytes = 0;
 
@@ -1157,6 +1516,11 @@ static void nd_dump_stats(void)
     json_object *json_devices = NULL;
     json_object *json_stats = NULL;
     json_object *json_flows = NULL;
+    json_object *json_data = NULL;
+#ifdef _ND_USE_PLUGINS
+    json_object *json_plugin_service_replies = NULL;
+    json_object *json_plugin_task_replies = NULL;
+#endif
 
     if (ND_USE_SINK || ND_JSON_SAVE) {
         json.AddObject(NULL, "version", (double)ND_JSON_VERSION);
@@ -1164,6 +1528,7 @@ static void nd_dump_stats(void)
         json.AddObject(NULL, "uptime",
             uint32_t(nda_stats.ts_now.tv_sec - nda_stats.ts_epoch.tv_sec));
         json.AddObject(NULL, "maxrss_kb", nda_stats.maxrss_kb);
+
 #if defined(_ND_USE_LIBTCMALLOC) && defined(HAVE_GPERFTOOLS_MALLOC_EXTENSION_H)
 #if (SIZEOF_LONG == 4)
         json.AddObject(NULL, "tcm_kb", (uint32_t)nda_stats.tcm_alloc_kb);
@@ -1171,20 +1536,16 @@ static void nd_dump_stats(void)
         json.AddObject(NULL, "tcm_kb", (uint64_t)nda_stats.tcm_alloc_kb);
 #endif
 #endif // _ND_USE_LIBTCMALLOC
-#ifndef _ND_LEAN_AND_MEAN
-        nd_sha1_to_string(nd_config.digest_content_match, digest);
-        json.AddObject(NULL, "content_match_digest", digest);
-        nd_sha1_to_string(nd_config.digest_host_match, digest);
-        json.AddObject(NULL, "host_match_digest", digest);
-#endif
-        nd_sha1_to_string(nd_config.digest_custom_match, digest);
-        json.AddObject(NULL, "custom_match_digest", digest);
 
         json_ifaces = json.CreateObject(NULL, "interfaces");
         json_devices = json.CreateObject(NULL, "devices");
         json_stats = json.CreateObject(NULL, "stats");
         json_flows = json.CreateObject(NULL, "flows");
-
+        json_data = json.CreateObject(NULL, "data");
+#ifdef _ND_USE_PLUGINS
+        json_plugin_service_replies = json.CreateObject(NULL, "service_replies");
+        json_plugin_task_replies = json.CreateObject(NULL, "task_replies");
+#endif
         nd_json_add_interfaces(json_ifaces);
         nd_json_add_devices(json_devices);
     }
@@ -1218,13 +1579,19 @@ static void nd_dump_stats(void)
         i->second->Unlock();
     }
 
+#ifdef _ND_USE_PLUGINS
+    nd_json_add_plugin_replies(
+        json_plugin_service_replies, json_plugin_task_replies, json_data
+    );
+#endif
+
     if (ND_USE_SINK) {
         try {
 #ifdef _ND_USE_INOTIFY
             for (nd_inotify_watch::const_iterator i = inotify_watches.begin();
                 i != inotify_watches.end(); i++) {
                 if (! inotify->EventOccured(i->first)) continue;
-                nd_json_add_file(json.GetRoot(), i->first, i->second);
+                nd_json_add_file(json_data, i->first, i->second);
             }
 #endif
             string json_string;
@@ -1232,7 +1599,7 @@ static void nd_dump_stats(void)
 #ifdef _ND_USE_WATCHDOGS
             nd_touch(ND_WD_UPLOAD);
 #endif
-            thread_upload->QueuePush(json_string);
+            thread_sink->QueuePush(json_string);
         }
         catch (runtime_error &e) {
             nd_printf("Error pushing JSON payload to upload queue: %s\n", e.what());
@@ -1251,11 +1618,14 @@ static void nd_dump_stats(void)
     json.Destroy();
 
     if (ND_DEBUG) {
+#ifndef _ND_LEAN_AND_MEAN
         if (ND_DEBUG_WITH_ETHERS) nd_load_ethers();
+#endif
         nd_print_stats();
     }
 }
 
+#ifndef _ND_LEAN_AND_MEAN
 static void nd_dump_protocols(void)
 {
     uint32_t custom_proto_base;
@@ -1269,8 +1639,9 @@ static void nd_dump_protocols(void)
         printf("%4d: %s\n", i, ndpi->proto_defaults[i].proto_name);
 
     ndpi_free(ndpi);
-    ndpi_global_init();
+    ndpi_global_destroy();
 }
+#endif
 
 #ifdef _ND_USE_NETLINK
 static void nd_add_device_addresses(nd_device_addr &device_addresses)
@@ -1445,8 +1816,6 @@ int main(int argc, char *argv[])
     static struct option options[] =
     {
         { "config", 1, 0, 'c' },
-        { "content-match", 1, 0, 'C' },
-        { "custom-match", 1, 0, 'f' },
         { "debug", 0, 0, 'd' },
         { "debug-dns-cache", 0, 0, 's' },
         { "debug-ether-names", 0, 0, 'e' },
@@ -1459,7 +1828,6 @@ int main(int argc, char *argv[])
         { "external", 1, 0, 'E' },
         { "hash-file", 1, 0, 'S' },
         { "help", 0, 0, 'h' },
-        { "host-match", 1, 0, 'H' },
         { "internal", 1, 0, 'I' },
         { "interval", 1, 0, 'i' },
         { "json", 1, 0, 'j' },
@@ -1468,6 +1836,8 @@ int main(int argc, char *argv[])
         { "remain-in-foreground", 0, 0, 'R' },
         { "replay-delay", 0, 0, 'r' },
         { "serial", 1, 0, 's' },
+        { "sink-config", 1, 0, 'f' },
+        { "test-output", 1, 0, 'T' },
         { "uuid", 1, 0, 'u' },
         { "uuidgen", 0, 0, 'U' },
         { "version", 0, 0, 'V' },
@@ -1481,7 +1851,7 @@ int main(int argc, char *argv[])
     for (optind = 1;; ) {
         int o = 0;
         if ((rc = getopt_long(argc, argv,
-            "?A:aC:c:DdE:eF:f:H:hI:i:j:lN:PpRrS:s:tUu:V",
+            "?A:ac:DdE:eF:f:hI:i:j:lN:PpRrS:s:tT:Uu:V",
             options, &o)) == -1) break;
         switch (rc) {
         case 0:
@@ -1512,11 +1882,6 @@ int main(int argc, char *argv[])
         case 'D':
             nd_config.flags |= ndGF_DEBUG_UPLOAD;
             break;
-        case 'C':
-            free(nd_config.path_content_match);
-            nd_config.path_content_match = strdup(optarg);
-            nd_config.flags |= ndGF_OVERRIDE_CONTENT_MATCH;
-            break;
         case 'E':
             for (nd_ifaces::iterator i = ifaces.begin();
                 i != ifaces.end(); i++) {
@@ -1544,14 +1909,9 @@ int main(int argc, char *argv[])
             nd_config.device_filters[last_device] = optarg;
             break;
         case 'f':
-            free(nd_config.path_custom_match);
-            nd_config.path_custom_match = strdup(optarg);
-            nd_config.flags |= ndGF_OVERRIDE_CUSTOM_MATCH;
-            break;
-        case 'H':
-            free(nd_config.path_host_match);
-            nd_config.path_host_match = strdup(optarg);
-            nd_config.flags |= ndGF_OVERRIDE_HOST_MATCH;
+            free(nd_config.path_sink_config);
+            nd_config.path_sink_config = strdup(optarg);
+            nd_config.flags |= ndGF_OVERRIDE_SINK_CONFIG;
             break;
         case 'h':
             nd_usage();
@@ -1588,8 +1948,13 @@ int main(int argc, char *argv[])
 #endif
             break;
         case 'P':
+#ifndef _ND_LEAN_AND_MEAN
             nd_dump_protocols();
             exit(0);
+#else
+            fprintf(stderr, "Sorry, not available from this build profile.\n");
+            exit(1);
+#endif
         case 'p':
             if (nd_conf_filename == NULL)
                 nd_conf_filename = strdup(ND_CONF_FILE_NAME);
@@ -1607,6 +1972,7 @@ int main(int argc, char *argv[])
             nd_config.flags |= ndGF_REPLAY_DELAY;
             break;
         case 'S':
+#ifndef _ND_LEAN_AND_MEAN
             {
                 uint8_t digest[SHA1_DIGEST_LENGTH];
 
@@ -1618,13 +1984,23 @@ int main(int argc, char *argv[])
                     return 0;
                 }
             }
-            break;
+#else
+            fprintf(stderr, "Sorry, not available from this build profile.\n");
+            exit(1);
+#endif
         case 's':
             nd_config.uuid_serial = strdup(optarg);
             break;
         case 't':
             nd_config.flags &= ~ndGF_USE_CONNTRACK;
             break;
+        case 'T':
+            if ((nd_config.h_flow = fopen(optarg, "w")) == NULL) {
+                fprintf(stderr, "Error while opening test output log: %s: %s\n",
+                    optarg, strerror(errno));
+                exit(1);
+            }
+            break;
         case 'U':
             {
                 string uuid;
@@ -1651,6 +2027,21 @@ int main(int argc, char *argv[])
     if (nd_config_load() < 0)
         return 1;
 
+    if (nd_config.h_flow != stderr) {
+        // Test mode enabled, disable/set certain config parameters
+        ND_GF_SET_FLAG(ndGF_USE_DNS_CACHE, true);
+        ND_GF_SET_FLAG(ndGF_DNS_CACHE_SAVE, false);
+        ND_GF_SET_FLAG(ndGF_USE_SINK, false);
+        ND_GF_SET_FLAG(ndGF_JSON_SAVE, false);
+        ND_GF_SET_FLAG(ndGF_REMAIN_IN_FOREGROUND, true);
+
+        nd_config.update_interval = 1;
+#ifdef _ND_USE_PLUGINS
+        nd_config.services.clear();
+        nd_config.tasks.clear();
+#endif
+    }
+
     if (ifaces.size() == 0) {
         fprintf(stderr,
             "Required argument, (-I, --internal, or -E, --external) missing.\n");
@@ -1671,17 +2062,30 @@ int main(int argc, char *argv[])
     }
 
     nd_printf("%s\n", nd_get_version_and_features().c_str());
+
     nd_check_agent_uuid();
+#ifndef _ND_LEAN_AND_MEAN
     nd_debug_printf("Flow entry size: %lu\n", sizeof(struct ndFlow) +
         sizeof(struct ndpi_flow_struct) + sizeof(struct ndpi_id_struct) * 2);
-
+#endif
     if (! ND_DEBUG) {
         FILE *hpid = fopen(ND_PID_FILE_NAME, "w+");
-        if (hpid == NULL) {
-            nd_printf("Error opening PID file: %s: %s\n",
-                ND_PID_FILE_NAME, strerror(errno));
-            return 1;
-        }
+
+        do {
+            if (hpid == NULL) {
+                nd_printf("Error opening PID file: %s: %s\n",
+                    ND_PID_FILE_NAME, strerror(errno));
+
+                if (mkdir(ND_VOLATILE_STATEDIR, 0755) == 0)
+                    hpid = fopen(ND_PID_FILE_NAME, "w+");
+                else {
+                    nd_printf("Unable to create volatile state directory: %s: %s\n",
+                        ND_VOLATILE_STATEDIR, strerror(errno));
+                    return 1;
+                }
+            }
+        } while(hpid == NULL);
+
         fprintf(hpid, "%d\n", getpid());
         fclose(hpid);
     }
@@ -1694,15 +2098,11 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    if (ND_USE_DNS_CACHE) dns_cache.load();
-#ifndef _ND_LEAN_AND_MEAN
+    if (ND_USE_DNS_CACHE && ND_DNS_CACHE_SAVE) dns_cache.load();
+
     nd_sha1_file(
-        nd_config.path_content_match, nd_config.digest_content_match);
-    nd_sha1_file(
-        nd_config.path_host_match, nd_config.digest_host_match);
-#endif
-    nd_sha1_file(
-        nd_config.path_custom_match, nd_config.digest_custom_match);
+        nd_config.path_sink_config, nd_config.digest_sink_config
+    );
 
     sigfillset(&sigset);
     //sigdelset(&sigset, SIGPROF);
@@ -1716,8 +2116,13 @@ int main(int argc, char *argv[])
     sigaddset(&sigset, ND_SIG_UPDATE);
     sigaddset(&sigset, SIGIO);
     sigaddset(&sigset, SIGHUP);
+    sigaddset(&sigset, SIGALRM);
+    sigaddset(&sigset, SIGUSR1);
+    sigaddset(&sigset, SIGUSR2);
 
-    nd_load_ethers();
+#ifndef _ND_LEAN_AND_MEAN
+    if (ND_DEBUG_WITH_ETHERS) nd_load_ethers();
+#endif
 
     try {
 #ifdef _ND_USE_CONNTRACK
@@ -1730,11 +2135,11 @@ int main(int argc, char *argv[])
             thread_socket = new ndSocketThread();
 
         if (ND_USE_SINK) {
-            thread_upload = new ndUploadThread();
-            thread_upload->Create();
+            thread_sink = new ndSinkThread();
+            thread_sink->Create();
         }
     }
-    catch (ndUploadThreadException &e) {
+    catch (ndSinkThreadException &e) {
         nd_printf("Error starting upload thread: %s\n", e.what());
         return 1;
     }
@@ -1808,16 +2213,8 @@ int main(int argc, char *argv[])
     }
 
 #ifdef _ND_USE_PLUGINS
-    try {
-        for (nd_plugin::const_iterator i = nd_config.plugins.begin();
-            i != nd_config.plugins.end(); i++) {
-            i->second->GetPlugin()->Create();
-        }
-    }
-    catch (ndThreadException &e) {
-        nd_printf("Error starting socket thread: %s\n", e.what());
+    if (nd_start_services() < 0)
         return 1;
-    }
 #endif
 
     memset(&sigev, 0, sizeof(struct sigevent));
@@ -1870,6 +2267,9 @@ int main(int argc, char *argv[])
             inotify->RefreshWatches();
 #endif
             nd_dump_stats();
+#ifdef _ND_USE_PLUGINS
+            nd_reap_tasks();
+#endif
             if (thread_socket) {
                 string json;
                 nd_json_agent_status(json);
@@ -1885,8 +2285,8 @@ int main(int argc, char *argv[])
             nd_reap_detection_threads();
 
             if (threads.size() == 0) {
-                if (thread_upload == NULL ||
-                    thread_upload->QueuePendingSize() == 0) {
+                if (thread_sink == NULL ||
+                    thread_sink->QueuePendingSize() == 0) {
                     nd_printf("Exiting, no remaining detection threads.\n");
                     terminate = true;
                     continue;
@@ -1916,16 +2316,25 @@ int main(int argc, char *argv[])
             continue;
         }
 
+        if (sig == SIGALRM) {
+            if (ND_USE_SINK && nd_sink_process_responses() < 0) break;
+
+            continue;
+        }
+
         if (sig == SIGHUP) {
+            continue;
+        }
+
+        if (sig == SIGUSR1) {
+            nd_start_detection_threads();
+            nd_detection_stopped_by_signal = false;
+            continue;
+        }
+
+        if (sig == SIGUSR2) {
             nd_stop_detection_threads();
-            if (nd_start_detection_threads() < 0) break;
-
-            if (thread_socket) {
-                string json;
-                nd_json_protocols(json);
-                thread_socket->QueueWrite(json);
-            }
-
+            nd_detection_stopped_by_signal = true;
             continue;
         }
 
@@ -1936,21 +2345,30 @@ int main(int argc, char *argv[])
 
     nd_stop_detection_threads();
 
-    if (thread_upload) {
-        thread_upload->Terminate();
-        delete thread_upload;
+#ifdef _ND_USE_PLUGINS
+    nd_stop_services();
+
+    nd_stop_tasks();
+    nd_reap_tasks();
+#endif
+
+    if (thread_sink) {
+        thread_sink->Terminate();
+        delete thread_sink;
     }
 
     if (thread_socket) {
         thread_socket->Terminate();
         delete thread_socket;
     }
+
 #ifdef _ND_USE_CONNTRACK
-    if (ND_USE_CONNTRACK) {
+    if (ND_USE_CONNTRACK && thread_conntrack) {
         thread_conntrack->Terminate();
         delete thread_conntrack;
     }
 #endif
+
     if (ND_USE_DNS_CACHE && ND_DNS_CACHE_SAVE)
         dns_cache.save();
     pthread_mutex_destroy(&dns_cache.lock);
@@ -1962,6 +2380,8 @@ int main(int argc, char *argv[])
 
     closelog();
 
+    if (! ND_DEBUG) unlink(ND_PID_FILE_NAME);
+
     return 0;
 }
 
